@@ -1,4 +1,4 @@
-﻿// Copyright 2013-2023 Daniel Parker
+﻿// Copyright 2013-2024 Daniel Parker
 // Distributed under Boost license
 
 #if defined(_MSC_VER)
@@ -13,7 +13,7 @@
 
 using namespace jsoncons;
 
-#if defined(JSONCONS_HAS_STATEFUL_ALLOCATOR)
+#if defined(JSONCONS_HAS_STATEFUL_ALLOCATOR) && JSONCONS_HAS_STATEFUL_ALLOCATOR == 1
 
 template<typename T>
 using MyScopedAllocator = std::scoped_allocator_adaptor<FreeListAllocator<T>>;
@@ -102,7 +102,7 @@ TEST_CASE("jsonpath stateful allocator test")
             {
                 CHECK((title.as<jsoncons::string_view>() == "Sword of Honour")); 
             }
-        );
+ );
     }
 
     SECTION("json_replace 1")
@@ -147,7 +147,7 @@ TEST_CASE("jsonpath stateful allocator test")
             {
                 price = std::round(price.as<double>() - 1.0); 
             }
-        );
+ );
 
         CHECK(8.0 == Approx(j["store"]["book"][0]["price"].as<double>()).epsilon(0.001));
         CHECK(12.0 == Approx(j["store"]["book"][1]["price"].as<double>()).epsilon(0.001));
